@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
+#include <libglom/libglom_config.h> // For GLOM_ENABLE_CLIENT_ONLY
 
 #include "box_data_details_find.h"
 #include <glibmm/i18n.h>
@@ -29,12 +29,12 @@ namespace Glom
 Box_Data_Details_Find::Box_Data_Details_Find()
 : Box_Data_Details(false)
 {
-   //m_strHint = _("Enter the search criteria and click [Find]\n Glom will then change to Data mode to display the results.");
-
   //Instead of nav buttons:
-  m_HBox.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
+  #ifndef GLOM_ENABLE_MAEMO //TODO_Maemo: Really have this buttons somewhere?
+  m_hbox_buttons.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
+  #endif
 
-  g_object_set(m_Button_Find.gobj(), "can-default", TRUE, (gpointer)NULL); //TODO: Make this a real method in gtkmm?
+  g_object_set(m_Button_Find.gobj(), "can-default", TRUE, (gpointer)0); //TODO: Make this a real method in gtkmm?
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   //Hide this because it is useless for Find mode:
@@ -48,16 +48,16 @@ Box_Data_Details_Find::~Box_Data_Details_Find()
 {
 }
 
-bool Box_Data_Details_Find::init_db_details(const Glib::ustring& table_name)
+bool Box_Data_Details_Find::init_db_details(const Glib::ustring& table_name, const Glib::ustring& layout_platform)
 {
   FoundSet found_set;
   found_set.m_table_name = table_name;
-  return Box_Data_Details::init_db_details(found_set, Gnome::Gda::Value());
+  return Box_Data_Details::init_db_details(found_set, layout_platform, Gnome::Gda::Value());
 }
 
 bool Box_Data_Details_Find::fill_from_database()
 {
-  Bakery::BusyCursor busy_cursor(get_app_window());
+  BusyCursor busy_cursor(get_app_window());
 
   const bool result = Base_DB_Table_Data::fill_from_database();
   if(!result)
@@ -82,10 +82,10 @@ void Box_Data_Details_Find::fill_related()
   //m_Notebook_Related.pages().clear();
 
   //Get relationships from the document:
-  Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_table_name);
+  Document::type_vec_relationships vecRelationships = get_document()->get_relationships(m_table_name);
 
   //Add the relationships:
-  for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
+  for(Document::type_vec_relationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
   {
      /*
      const Relationship& relationship = *iter;
