@@ -18,6 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include "datawidget.h"
 #include "entryglom.h"
 #include "checkglom.h"
@@ -27,13 +28,18 @@
 #include "textviewglom.h"
 #include "imageglom.h"
 #include <libglom/data_structure/glomconversions.h>
-#include "../application.h"
-#include "../mode_data/dialog_choose_field.h"
+#include <glom/application.h>
+#include <glom/mode_design/layout/dialog_choose_field.h>
 #include "dialog_choose_id.h"
 #include "dialog_choose_date.h"
-#include "../layout_item_dialogs/dialog_field_layout.h"
-#include <libglom/utils.h>
-#include <libglom/glade_utils.h>
+#include <glom/mode_design/layout/layout_item_dialogs/dialog_field_layout.h>
+#include <glom/utils_ui.h>
+#include <glom/glade_utils.h>
+
+#ifdef GLOM_ENABLE_MAEMO
+#include <hildonmm/button.h>
+#endif //GLOM_ENABLE_MAEMO
+
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -45,7 +51,9 @@ DataWidget::DataWidget(Field::glom_field_type glom_type, const Glib::ustring& ti
   m_pMenuPopup(0)
 */
 
-DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ustring& table_name, const Document_Glom* document)
+DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ustring& table_name, const Document* document)
+:  m_child(0),
+   m_button_go_to_details(0)
 {
   const Field::glom_field_type glom_type = field->get_glom_type();
   set_layout_item(field, table_name);
@@ -230,7 +238,11 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     if(glom_type == Field::TYPE_DATE)
     {
       //Let the user choose a date from a calendar dialog:
-      Gtk::Button* button_date = Gtk::manage(new Gtk::Button(_("..."))); //TODO: A better label/icon for "Choose Date". 
+      #ifndef GLOM_ENABLE_MAEMO
+      Gtk::Button* button_date = Gtk::manage(new Gtk::Button(_("..."))); //TODO: A better label/icon for "Choose Date".
+      #else
+      Gtk::Button* button_date = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT, Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("..."), ""));
+      #endif
       button_date->set_tooltip_text(_("Choose a date from an on-screen calendar.")); 
       button_date->show();
       hbox_parent->pack_start(*button_date);
@@ -240,17 +252,26 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     if((field_used_in_relationship_to_one || field_is_related_primary_key) && hbox_parent)
     {
       //Add a button for related record navigation:
-      Gtk::Button* button_go_to_details = Gtk::manage(new Gtk::Button(Gtk::Stock::OPEN));
-      button_go_to_details->set_tooltip_text(_("Open the record identified by this ID, in the other table."));
-      hbox_parent->pack_start(*button_go_to_details);
-      button_go_to_details->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_open_details));
+      #ifndef GLOM_ENABLE_MAEMO
+      m_button_go_to_details = Gtk::manage(new Gtk::Button(Gtk::Stock::OPEN));
+      #else
+      m_button_go_to_details = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT, Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("Open"), ""));
+      #endif
+      m_button_go_to_details->set_tooltip_text(_("Open the record identified by this ID, in the other table."));
+      hbox_parent->pack_start(*m_button_go_to_details);
+      m_button_go_to_details->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_open_details));
 
       //Add a button to make it easier to choose an ID for this field.
       //Don't add this for simple related primary key fields, because they 
       //can generally not be edited via another table's layout.
       if(field_used_in_relationship_to_one)
       {
+        #ifndef GLOM_ENABLE_MAEMO
         Gtk::Button* button_select = Gtk::manage(new Gtk::Button(Gtk::Stock::FIND));
+        #else
+        Gtk::Button* button_select = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT, 
+           Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("Find"), ""));
+        #endif
         button_select->set_tooltip_text(_("Enter search criteria to identify records in the other table, to choose an ID for this field."));
         hbox_parent->pack_start(*button_select);
         button_select->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_select_id));
@@ -276,6 +297,7 @@ DataWidget::~DataWidget()
 void DataWidget::on_widget_edited()
 {
   m_signal_edited.emit(get_value());
+  update_go_to_details_button_sensitivity();
 }
 
 DataWidget::type_signal_edited DataWidget::signal_edited()
@@ -307,6 +329,20 @@ void DataWidget::set_value(const Gnome::Gda::Value& value)
       checkbutton->set_active( bValue );
     }
   }
+
+  update_go_to_details_button_sensitivity();
+}
+
+void DataWidget::update_go_to_details_button_sensitivity()
+{
+  //If there is a Go-To-Details "Open" button, only enable it if there is 
+  //an ID:
+  if(m_button_go_to_details)
+  {
+    const Gnome::Gda::Value value = get_value();
+    const bool enabled = !Conversions::value_is_empty(value);
+    m_button_go_to_details->set_sensitive(enabled);
+  }
 }
 
 Gnome::Gda::Value DataWidget::get_value() const
@@ -340,7 +376,7 @@ const Gtk::Label* DataWidget::get_label() const
 void DataWidget::set_child_size_by_field(const sharedptr<const LayoutItem_Field>& field)
 {
   const Field::glom_field_type glom_type = field->get_glom_type();
-  const int width = get_suitable_width(field);
+  int width = get_suitable_width(field);
 
   if(glom_type == Field::TYPE_IMAGE) //GtkImage widgets default to no size (invisible) if they are empty.
     m_child->set_size_request(width, width);
@@ -349,12 +385,21 @@ void DataWidget::set_child_size_by_field(const sharedptr<const LayoutItem_Field>
     int height = -1; //auto.
     if((glom_type == Field::TYPE_TEXT) && (field->get_formatting_used().get_text_format_multiline()))
     {
+      #ifndef GLOM_ENABLE_MAEMO 
       int example_width = 0;
       int example_height = 0;
       Glib::RefPtr<Pango::Layout> refLayout = create_pango_layout("example"); //TODO: Use different text, according to the current locale, or allow the user to choose an example?
       refLayout->get_pixel_size(example_width, example_height);
+      
       if(example_height > 0)
         height = example_height * field->get_formatting_used().get_text_format_multiline_height_lines();
+      #else
+      //On Maemo, TextView widgets expand automatically.
+      //TODO: Expansion only happens if both are -1, and vertical expansion never happens.
+      //See bug https://bugs.maemo.org/show_bug.cgi?id=5515
+      width = -1;
+      height = -1; 
+      #endif //GLOM_ENABLE_MAEMO
     }
 
     m_child->set_size_request(width, height);
@@ -363,71 +408,7 @@ void DataWidget::set_child_size_by_field(const sharedptr<const LayoutItem_Field>
 
 int DataWidget::get_suitable_width(const sharedptr<const LayoutItem_Field>& field_layout)
 {
-  int result = 150;
-
-  const Field::glom_field_type field_type = field_layout->get_glom_type();
-
-  Glib::ustring example_text;
-  switch(field_type)
-  {
-    case(Field::TYPE_DATE):
-    {
-      Glib::Date date(31, Glib::Date::Month(12), 2000);
-      example_text = Conversions::get_text_for_gda_value(field_type, Gnome::Gda::Value(date));
-      break;
-    }
-    case(Field::TYPE_TIME):
-    {
-      Gnome::Gda::Time time = {0, 0, 0, 0, 0};
-      time.hour = 24;
-      time.minute = 59;
-      time.second = 59;
-      example_text = Conversions::get_text_for_gda_value(field_type, Gnome::Gda::Value(time));
-      break;
-    }
-    case(Field::TYPE_NUMERIC):
-    {
-#ifdef GLOM_ENABLE_MAEMO
-      //Maemo's screen is not so big, so don't be so generous:
-      example_text = "EUR 9999999";
-#else
-      example_text = "EUR 9999999999";
-#endif
-      break;
-    }
-    case(Field::TYPE_TEXT):
-    case(Field::TYPE_IMAGE): //Give images the same width as text fields, so they will often line up.
-    {
-      //if(!field_layout->get_text_format_multiline()) //Use the full width for multi-line text.
-#ifdef GLOM_ENABLE_MAEMO
-        //Maemo's screen is not so big, so don't be so generous:
-        example_text = "AAAAAAAAAAAAAAAA";
-#else
-        example_text = "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
-#endif
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
-
-
-  if(!example_text.empty())
-  {
-    //Get the width required for this string in the current font:
-    Glib::RefPtr<Pango::Layout> refLayout = create_pango_layout(example_text);
-    int width = 0;
-    int height = 0;
-    refLayout->get_pixel_size(width, height);
-    result = width;
-
-    //Add a bit more:
-    result += 10;
-  }
-
-  return result;
+  return Utils::get_suitable_field_width_for_widget(*this, field_layout);
 }
 
 void DataWidget::set_viewable(bool viewable)
@@ -572,7 +553,6 @@ bool DataWidget::on_button_press_event(GdkEventButton *event)
 
   return Gtk::EventBox::on_button_press_event(event);
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name)
 {
@@ -581,33 +561,33 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& ta
 
 sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field)
 {
-  return offer_field_list (table_name, start_field, get_document(), get_application());
+  return offer_field_list(table_name, start_field, get_document(), get_application());
 }
 
-sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field, Document_Glom* document, App_Glom* app)
+sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field, Document* document, App_Glom* app)
 {
   sharedptr<LayoutItem_Field> result;
 
-  Glib::RefPtr<Gnome::Glade::Xml> refXml;
+  Glib::RefPtr<Gtk::Builder> refXml;
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_field");
+    refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_field");
   }
-  catch(const Gnome::Glade::XmlError& ex)
+  catch(const Gtk::BuilderError& ex)
   {
     std::cerr << ex.what() << std::endl;
     return result;
   }
 #else
-  std::auto_ptr<Gnome::Glade::XmlError> error;
-  refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_field", "", error);
-  if(error.get())
+  std::auto_ptr<Glib::Error> error;
+  refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_field", error);
+  if (error.get())
   {
     std::cerr << error->what() << std::endl;
     return result;
-  }
+  }  
 #endif
   
   Dialog_ChooseField* dialog = 0;
@@ -631,14 +611,13 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& ta
   return result;
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 sharedptr<LayoutItem_Field> DataWidget::offer_field_layout(const sharedptr<const LayoutItem_Field>& start_field)
 {
   sharedptr<LayoutItem_Field> result;
 
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_layout_field_properties");
+    Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_layout_field_properties");
 
     Dialog_FieldLayout* dialog = 0;
     refXml->get_widget_derived("dialog_layout_field_properties", dialog);
@@ -660,23 +639,14 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_layout(const sharedptr<const
       delete dialog;
     }
   }
-  catch(const Gnome::Glade::XmlError& ex)
+  catch(const Gtk::BuilderError& ex)
   {
     std::cerr << ex.what() << std::endl;
   }
 
   return result;
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-/*
-void DataWidget::on_menupopup_add_item(LayoutWidgetBase::enumType item)
-{
-  signal_layout_item_added().emit(item);
-}
-*/
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 void DataWidget::on_menupopup_activate_layout()
 {
   //finish_editing();
@@ -771,25 +741,25 @@ const Gtk::Widget* DataWidget::get_data_child_widget() const
 
 void DataWidget::on_button_choose_date()
 {
-  Glib::RefPtr<Gnome::Glade::Xml> refXml;
+  Glib::RefPtr<Gtk::Builder> refXml;
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_date");
+    refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_date");
   }
-  catch(const Gnome::Glade::XmlError& ex)
+  catch(const Gtk::BuilderError& ex)
   {
     std::cerr << ex.what() << std::endl;
     return;
   }
 #else
-  std::auto_ptr<Gnome::Glade::XmlError> error;
-  refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_date", "", error);
-  if(error.get())
+  std::auto_ptr<Glib::Error> error;
+  refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_date", error);
+  if (error.get())
   {
     std::cerr << error->what() << std::endl;
     return;
-  }
+  } 
 #endif
 
   Dialog_ChooseDate* dialog = 0;
@@ -828,21 +798,21 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
   //Initialize output variable:
   chosen_id = Gnome::Gda::Value();
 
-  Glib::RefPtr<Gnome::Glade::Xml> refXml;
+  Glib::RefPtr<Gtk::Builder> refXml;
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom.glade"), "dialog_find_id");
+    refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom.glade"), "dialog_find_id");
   }
-  catch(const Gnome::Glade::XmlError& ex)
+  catch(const Glib::Error& ex)
   {
     std::cerr << ex.what() << std::endl;
     return result;
   }
 #else
-  std::auto_ptr<Gnome::Glade::XmlError> error;
-  refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom.glade"), "dialog_find_id", "", error);
-  if(error.get())
+  std::auto_ptr<Glib::Error> error;
+  refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom.glade"), "dialog_find_id", error);
+  if (error.get())
   {
     std::cerr << error->what() << std::endl;
     return result;
@@ -870,7 +840,7 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
     else
       g_warning("get_layout_item() was not a LayoutItem_Field");
 
-    dialog->init_db_details(related_table_name, get_document()->get_active_layout_platform());
+    dialog->init_db_details(related_table_name, Base_DB::get_active_layout_platform(get_document()));
 
 
     const int response = dialog->run();
